@@ -148,10 +148,10 @@ servicelog_notify_log(servicelog *slog, struct sl_notify *notify,
 
 	rc = sqlite3_bind_int(pstmt, 1,	notify->notify);
 	rc = rc ? rc : sqlite3_bind_text(pstmt, 2, notify->command,
-					 strlen(notify->command), SQLITE_STATIC);
+		notify->command ? strlen(notify->command):0, SQLITE_STATIC);
 	rc = rc ? rc : sqlite3_bind_int(pstmt, 3, notify->method);
 	rc = rc ? rc : sqlite3_bind_text(pstmt, 4, notify->match,
-					 strlen(notify->match), SQLITE_STATIC);
+			notify->match ? strlen(notify->match):0, SQLITE_STATIC);
 	if (rc != SQLITE_OK)
 		goto sqlt_fail;
 
@@ -261,6 +261,8 @@ servicelog_notify_query(servicelog *slog, char *query,
 				n->notify = sqlite3_column_int(stmt, i);
 			else if (!strcmp(name, "command")) {
 				str = (char *)sqlite3_column_text(stmt, i);
+				if (!str)
+					str = "";
 				n->command = strdup(str);
 				if (!n->command)
 					goto free_mem;
@@ -269,6 +271,8 @@ servicelog_notify_query(servicelog *slog, char *query,
 				n->method = sqlite3_column_int(stmt, i);
 			else if (!strcmp(name, "match")) {
 				str = (char *)sqlite3_column_text(stmt, i);
+				if (!str)
+					str = "";
 				n->match = strdup(str);
 				if (!n->match)
 					goto free_mem;
@@ -341,10 +345,10 @@ servicelog_notify_update(servicelog *slog, uint64_t notify_id,
 
 	rc = sqlite3_bind_int(pstmt, 1, notify->notify);
 	rc = rc ? rc : sqlite3_bind_text(pstmt, 2, notify->command,
-					 strlen(notify->command), SQLITE_STATIC);
+		notify->command ? strlen(notify->command):0, SQLITE_STATIC);
 	rc = rc ? rc : sqlite3_bind_int(pstmt, 3, notify->method);
 	rc = rc ? rc : sqlite3_bind_text(pstmt, 4, notify->match,
-					 strlen(notify->match), SQLITE_STATIC);
+			notify->match ? strlen(notify->match):0, SQLITE_STATIC);
 	rc = rc ? rc : sqlite3_bind_int64(pstmt, 5, notify_id);
 	if (rc != SQLITE_OK)
 		goto sqlt_fail;
@@ -499,7 +503,8 @@ run_notification_tool(struct sl_notify *notify, int type, void *records)
 	struct sl_repair_action *repair = NULL;
 	void *record = records;
 	uint64_t id = 0;
-	int pipe_fd[2], pid, i;
+	int pipe_fd[2], pid, i, rc = 0;
+	ssize_t __attribute__((unused)) rc1;
 	FILE *stream;
 
 	if (notify == NULL)
@@ -530,13 +535,21 @@ run_notification_tool(struct sl_notify *notify, int type, void *records)
 		}
 
 		pid = fork();
-
 		if (pid == -1) {	/* fork failed */
+			if (notify->method != SL_METHOD_NUM_VIA_CMD_LINE) {
+				close(pipe_fd[0]);
+				close(pipe_fd[1]);
+			}
 			continue;
-		}
-		else if (pid == 0) {	/* child; set up pipes, exec command */
-			close(pipe_fd[1]);
-			dup2(pipe_fd[0], fileno(stdin));
+		} else if (pid == 0) {	/* child; set up pipes, exec command */
+			if (notify->method != SL_METHOD_NUM_VIA_CMD_LINE) {
+				close(pipe_fd[1]);
+				rc = dup2(pipe_fd[0], fileno(stdin));
+				if (rc == -1) {
+					close(pipe_fd[0]);
+					return -1;
+				}
+			}
 
 			/* build up args for execv */
 			for (i = 0; i < 30; i++)
@@ -585,27 +598,33 @@ run_notification_tool(struct sl_notify *notify, int type, void *records)
 		if (notify->method == SL_METHOD_NUM_VIA_STDIN) {
 			close(pipe_fd[0]);
 			snprintf(buf, 128, "%" PRIu64, id);
-			write(pipe_fd[1], buf, strlen(buf));
+			rc1 = write(pipe_fd[1], buf, strlen(buf));
 			close(pipe_fd[1]);
 		}
 		else if (notify->method == SL_METHOD_PRETTY_VIA_STDIN) {
 			close(pipe_fd[0]);
 			stream = fdopen(pipe_fd[1], "w");
-			if (type == SL_NOTIFY_EVENTS)
-				servicelog_event_print(stream, event, 2);
-			else if (type == SL_NOTIFY_REPAIRS)
-				servicelog_repair_print(stream, repair, 2);
-			fclose(stream);
+			if (stream) {
+				if (type == SL_NOTIFY_EVENTS)
+					servicelog_event_print(stream, event, 2);
+				else if (type == SL_NOTIFY_REPAIRS)
+					servicelog_repair_print(stream, repair, 2);
+
+				fclose(stream);
+			}
 			close(pipe_fd[1]);
 		}
 		else if (notify->method == SL_METHOD_SIMPLE_VIA_STDIN) {
 			close(pipe_fd[0]);
 			stream = fdopen(pipe_fd[1], "w");
-			if (type == SL_NOTIFY_EVENTS)
-				servicelog_event_print(stream, event, -1);
-			else if (type == SL_NOTIFY_REPAIRS)
-				servicelog_repair_print(stream, repair, -1);
-			fclose(stream);
+			if (stream) {
+				if (type == SL_NOTIFY_EVENTS)
+					servicelog_event_print(stream, event, -1);
+				else if (type == SL_NOTIFY_REPAIRS)
+					servicelog_repair_print(stream,repair, -1);
+
+				fclose(stream);
+			}
 			close(pipe_fd[1]);
 		}
 
@@ -658,6 +677,10 @@ check_notify(void *d, int argc, char **argv, char **column)
 	memset(&notify, 0, sizeof(struct sl_notify));
 
 	for (i=0; i<argc; i++) {
+
+		if ((!argv[i]) || (!column[i]))
+			goto free_mem;
+
 		if (!strcmp(column[i], "id"))
 			notify.id = strtoull(argv[i], NULL, 10);
 		else if (!strcmp(column[i], "time_logged")) {
