@@ -244,6 +244,7 @@ sl_printf(FILE *stream, char *fmt, ...)
 	int tmpbuf_len, width = 0, prnt_len;
 	char buf[1024], tmpbuf[1024];
 	char *newline = NULL, *brkpt = NULL;
+	int size = sizeof(buf) - 1;
 
 	memset(tmpbuf, 0, sizeof(tmpbuf));
 	memset(buf, 0, sizeof(buf));
@@ -278,10 +279,12 @@ sl_printf(FILE *stream, char *fmt, ...)
 
 			if (newline != NULL) {
 				prnt_len = newline - &tmpbuf[offset] + 1;
-				snprintf(buf + buf_offset, prnt_len,
+				size -= buf_offset;
+				snprintf(buf + buf_offset, size,
 					 "%s", &tmpbuf[offset]);
 				buf_offset = strlen(buf);
-				buf_offset += sprintf(buf + buf_offset, "\n");
+				buf[*buf + buf_offset] = '\n';
+				buf_offset += 1;
 				offset += prnt_len;
 				line_offset = 0;
 				break;
@@ -297,15 +300,18 @@ sl_printf(FILE *stream, char *fmt, ...)
 			}
 
 			/* print up to the last brkpt */
-			snprintf(buf + buf_offset, prnt_len, "%s", &tmpbuf[offset]);
+			size -= buf_offset;
+			snprintf(buf + buf_offset, size, "%s", &tmpbuf[offset]);
 			buf_offset = strlen(buf);
-			buf_offset += sprintf(buf + buf_offset, "\n");
+			buf[*buf + buf_offset] = '\n';
+			buf_offset += 1;
 			offset += prnt_len;
 			line_offset = 0;
 		}
 	}
 
-	prnt_len = sprintf(buf + buf_offset, "%s", &tmpbuf[offset]);
+	size -= buf_offset;
+	prnt_len = snprintf(buf + buf_offset, size, "%s", &tmpbuf[offset]);
 	line_offset += prnt_len;
 
 	return fprintf(stream, "%s", buf);
@@ -324,7 +330,7 @@ size_t
 get_system_info(char *var, char *buf, size_t sz)
 {
 	int n_read, n_junk_chars = 0;
-	char filebuf[32], junk_chars[6];
+	char filebuf[32];
 	FILE *fd;
 
 	buf[0] = '\0';
@@ -344,8 +350,10 @@ get_system_info(char *var, char *buf, size_t sz)
 	if (fd == NULL)
 		return 0;
 
-	if (n_junk_chars > 0)
-		fread(junk_chars, n_junk_chars, 1, fd);
+	if (fseek(fd, n_junk_chars, SEEK_SET)) {
+		fclose(fd);
+		return 0;
+	}
 
 	n_read = fread(buf, sz, 1, fd);
 	fclose(fd);
@@ -421,30 +429,6 @@ replace_query_keywords(servicelog *slog, char *query, sqlite3_stmt **stmt,
 	return 0;
 }
 
-/**
- * format_text_to_insert
- * @brief Replace one ' (apostrophe) with two to make a legal SQL string.
- *
- * @param[in] input input string
- * @param[out] output legal SQL string
- * @param[in] size size of the output buffer
- */
-void
-format_text_to_insert(char *input, char *output, int size)
-{
-	char *end = output + size - 1; /* Leave room for trailing NULL. */
-
-	while (*input && output < end) {
-		if (*input == '\'') {
-			if (output + 2 > end)
-				break; /* Lack room for 2 apostrophes. */
-			*output++ = '\'';
-		}
-		*output++ = *input++;
-	}
-	*output = '\0';
-}
-
 /**
  * servicelog_init
  * @brief Populate the database with tables and triggers.
@@ -574,8 +558,9 @@ servicelog_open(servicelog **slog, uint32_t flags)
 
 	*slog = NULL;
 
-	log = malloc(sizeof(struct servicelog));
-	memset(log, 0, sizeof(struct servicelog));
+	log = calloc(1, sizeof(struct servicelog));
+	if (!log)
+		return ENOMEM;
 
 	log->flags = flags;
 	log->location = SERVICELOG_PATH;
@@ -626,36 +611,36 @@ int
 servicelog_truncate(servicelog *slog, int notifications_too)
 {
 	int rc;
+	char *err = NULL;
 
 	if (slog == NULL)
 		return EINVAL;
 
-	if ((slog->flags && SL_FLAG_ADMIN) == 0) {
+	if ((slog->flags & SL_FLAG_ADMIN) == 0) {
 		snprintf(slog->error, SL_MAX_ERR, "The database must be "
 			 "opened with the ADMIN flag");
 		return EACCES;
 	}
 
-	rc = sqlite3_exec(slog->db, "DELETE FROM events",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM callouts",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM os",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM rtas",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM enclosure",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM bmc",
-			  NULL, NULL, NULL);
-	rc = sqlite3_exec(slog->db, "DELETE FROM repair_actions",
-			  NULL, NULL, NULL);
+	rc = sqlite3_exec(slog->db, "DELETE FROM events", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM callouts", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM os", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM rtas", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM enclosure", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM bmc", NULL, NULL, &err);
+	rc = rc ? rc : sqlite3_exec(slog->db, "DELETE FROM repair_actions",
+							NULL, NULL, &err);
 
-	if (notifications_too)
+	if ((rc == SQLITE_OK) && notifications_too) {
 		rc = sqlite3_exec(slog->db, "DELETE FROM notifications",
-				  NULL, NULL, NULL);
+				  NULL, NULL, &err);
+	}
 
-	return 0;
+	if (rc != SQLITE_OK)
+		snprintf(slog->error, SL_MAX_ERR, "Error: %s", err);
+
+	sqlite3_free(err);
+	return rc;
 }
 
 /**
